# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.2.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _fifechan
else:
    import _fifechan

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _fifechan.SWIG_PyInstanceMethod_New
_swig_new_static_method = _fifechan.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_SwigPyIterator
    value = _swig_new_instance_method(_fifechan.SwigPyIterator_value)
    incr = _swig_new_instance_method(_fifechan.SwigPyIterator_incr)
    decr = _swig_new_instance_method(_fifechan.SwigPyIterator_decr)
    distance = _swig_new_instance_method(_fifechan.SwigPyIterator_distance)
    equal = _swig_new_instance_method(_fifechan.SwigPyIterator_equal)
    copy = _swig_new_instance_method(_fifechan.SwigPyIterator_copy)
    next = _swig_new_instance_method(_fifechan.SwigPyIterator_next)
    __next__ = _swig_new_instance_method(_fifechan.SwigPyIterator___next__)
    previous = _swig_new_instance_method(_fifechan.SwigPyIterator_previous)
    advance = _swig_new_instance_method(_fifechan.SwigPyIterator_advance)
    __eq__ = _swig_new_instance_method(_fifechan.SwigPyIterator___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.SwigPyIterator___ne__)
    __iadd__ = _swig_new_instance_method(_fifechan.SwigPyIterator___iadd__)
    __isub__ = _swig_new_instance_method(_fifechan.SwigPyIterator___isub__)
    __add__ = _swig_new_instance_method(_fifechan.SwigPyIterator___add__)
    __sub__ = _swig_new_instance_method(_fifechan.SwigPyIterator___sub__)
    def __iter__(self):
        return self

# Register SwigPyIterator in _fifechan:
_fifechan.SwigPyIterator_swigregister(SwigPyIterator)
class MouseListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_MouseListener
    mouseEntered = _swig_new_instance_method(_fifechan.MouseListener_mouseEntered)
    mouseExited = _swig_new_instance_method(_fifechan.MouseListener_mouseExited)
    mousePressed = _swig_new_instance_method(_fifechan.MouseListener_mousePressed)
    mouseReleased = _swig_new_instance_method(_fifechan.MouseListener_mouseReleased)
    mouseClicked = _swig_new_instance_method(_fifechan.MouseListener_mouseClicked)
    mouseWheelMovedUp = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedUp)
    mouseWheelMovedDown = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedDown)
    mouseWheelMovedRight = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedRight)
    mouseWheelMovedLeft = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedLeft)
    mouseMoved = _swig_new_instance_method(_fifechan.MouseListener_mouseMoved)
    mouseDragged = _swig_new_instance_method(_fifechan.MouseListener_mouseDragged)

    def __init__(self):
        if self.__class__ == MouseListener:
            _self = None
        else:
            _self = self
        _fifechan.MouseListener_swiginit(self, _fifechan.new_MouseListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_MouseListener(self)
        return weakref.proxy(self)

# Register MouseListener in _fifechan:
_fifechan.MouseListener_swigregister(MouseListener)
class KeyListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_KeyListener
    keyPressed = _swig_new_instance_method(_fifechan.KeyListener_keyPressed)
    keyReleased = _swig_new_instance_method(_fifechan.KeyListener_keyReleased)

    def __init__(self):
        if self.__class__ == KeyListener:
            _self = None
        else:
            _self = self
        _fifechan.KeyListener_swiginit(self, _fifechan.new_KeyListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_KeyListener(self)
        return weakref.proxy(self)

# Register KeyListener in _fifechan:
_fifechan.KeyListener_swigregister(KeyListener)
class ActionListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_ActionListener
    action = _swig_new_instance_method(_fifechan.ActionListener_action)

    def __init__(self):
        if self.__class__ == ActionListener:
            _self = None
        else:
            _self = self
        _fifechan.ActionListener_swiginit(self, _fifechan.new_ActionListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_ActionListener(self)
        return weakref.proxy(self)

# Register ActionListener in _fifechan:
_fifechan.ActionListener_swigregister(ActionListener)
class WidgetListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_WidgetListener
    widgetResized = _swig_new_instance_method(_fifechan.WidgetListener_widgetResized)
    widgetMoved = _swig_new_instance_method(_fifechan.WidgetListener_widgetMoved)
    widgetHidden = _swig_new_instance_method(_fifechan.WidgetListener_widgetHidden)
    widgetShown = _swig_new_instance_method(_fifechan.WidgetListener_widgetShown)
    ancestorMoved = _swig_new_instance_method(_fifechan.WidgetListener_ancestorMoved)
    ancestorHidden = _swig_new_instance_method(_fifechan.WidgetListener_ancestorHidden)
    ancestorShown = _swig_new_instance_method(_fifechan.WidgetListener_ancestorShown)

    def __init__(self):
        if self.__class__ == WidgetListener:
            _self = None
        else:
            _self = self
        _fifechan.WidgetListener_swiginit(self, _fifechan.new_WidgetListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_WidgetListener(self)
        return weakref.proxy(self)

# Register WidgetListener in _fifechan:
_fifechan.WidgetListener_swigregister(WidgetListener)
getVersion = _fifechan.getVersion
getMajor = _fifechan.getMajor
getMinor = _fifechan.getMinor
getPatch = _fifechan.getPatch
getHash = _fifechan.getHash
class Graphics(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    Left = _fifechan.Graphics_Left
    Center = _fifechan.Graphics_Center
    Right = _fifechan.Graphics_Right
    __swig_destroy__ = _fifechan.delete_Graphics

# Register Graphics in _fifechan:
_fifechan.Graphics_swigregister(Graphics)
class Color(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Color_swiginit(self, _fifechan.new_Color(*args))
    __add__ = _swig_new_instance_method(_fifechan.Color___add__)
    __sub__ = _swig_new_instance_method(_fifechan.Color___sub__)
    __mul__ = _swig_new_instance_method(_fifechan.Color___mul__)
    __eq__ = _swig_new_instance_method(_fifechan.Color___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.Color___ne__)
    r = property(_fifechan.Color_r_get, _fifechan.Color_r_set)
    g = property(_fifechan.Color_g_get, _fifechan.Color_g_set)
    b = property(_fifechan.Color_b_get, _fifechan.Color_b_set)
    a = property(_fifechan.Color_a_get, _fifechan.Color_a_set)
    __swig_destroy__ = _fifechan.delete_Color

# Register Color in _fifechan:
_fifechan.Color_swigregister(Color)
class Size(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, width=0, height=0):
        _fifechan.Size_swiginit(self, _fifechan.new_Size(width, height))
    __swig_destroy__ = _fifechan.delete_Size
    getWidth = _swig_new_instance_method(_fifechan.Size_getWidth)
    getHeight = _swig_new_instance_method(_fifechan.Size_getHeight)
    setWidth = _swig_new_instance_method(_fifechan.Size_setWidth)
    setHeight = _swig_new_instance_method(_fifechan.Size_setHeight)

# Register Size in _fifechan:
_fifechan.Size_swigregister(Size)
class Point(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_fifechan.Point_x_get, _fifechan.Point_x_set)
    y = property(_fifechan.Point_y_get, _fifechan.Point_y_set)

    def __init__(self, *args):
        _fifechan.Point_swiginit(self, _fifechan.new_Point(*args))
    __add__ = _swig_new_instance_method(_fifechan.Point___add__)
    __sub__ = _swig_new_instance_method(_fifechan.Point___sub__)
    __iadd__ = _swig_new_instance_method(_fifechan.Point___iadd__)
    __isub__ = _swig_new_instance_method(_fifechan.Point___isub__)
    __mul__ = _swig_new_instance_method(_fifechan.Point___mul__)

    def __truediv__(self, *args):
        return _fifechan.Point___truediv__(self, *args)
    __div__ = __truediv__


    __eq__ = _swig_new_instance_method(_fifechan.Point___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.Point___ne__)
    length = _swig_new_instance_method(_fifechan.Point_length)
    normalize = _swig_new_instance_method(_fifechan.Point_normalize)
    rotate = _swig_new_instance_method(_fifechan.Point_rotate)
    set = _swig_new_instance_method(_fifechan.Point_set)
    __swig_destroy__ = _fifechan.delete_Point

# Register Point in _fifechan:
_fifechan.Point_swigregister(Point)
class Widget(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    Selection_None = _fifechan.Widget_Selection_None
    Selection_Border = _fifechan.Widget_Selection_Border
    Selection_Background = _fifechan.Widget_Selection_Background
    getParent = _swig_new_instance_method(_fifechan.Widget_getParent)
    setWidth = _swig_new_instance_method(_fifechan.Widget_setWidth)
    getWidth = _swig_new_instance_method(_fifechan.Widget_getWidth)
    setHeight = _swig_new_instance_method(_fifechan.Widget_setHeight)
    getHeight = _swig_new_instance_method(_fifechan.Widget_getHeight)
    setSize = _swig_new_instance_method(_fifechan.Widget_setSize)
    setX = _swig_new_instance_method(_fifechan.Widget_setX)
    getX = _swig_new_instance_method(_fifechan.Widget_getX)
    setY = _swig_new_instance_method(_fifechan.Widget_setY)
    getY = _swig_new_instance_method(_fifechan.Widget_getY)
    setPosition = _swig_new_instance_method(_fifechan.Widget_setPosition)
    setOutlineSize = _swig_new_instance_method(_fifechan.Widget_setOutlineSize)
    getOutlineSize = _swig_new_instance_method(_fifechan.Widget_getOutlineSize)
    setBorderSize = _swig_new_instance_method(_fifechan.Widget_setBorderSize)
    getBorderSize = _swig_new_instance_method(_fifechan.Widget_getBorderSize)
    setMargin = _swig_new_instance_method(_fifechan.Widget_setMargin)
    setMarginTop = _swig_new_instance_method(_fifechan.Widget_setMarginTop)
    getMarginTop = _swig_new_instance_method(_fifechan.Widget_getMarginTop)
    setMarginRight = _swig_new_instance_method(_fifechan.Widget_setMarginRight)
    getMarginRight = _swig_new_instance_method(_fifechan.Widget_getMarginRight)
    setMarginBottom = _swig_new_instance_method(_fifechan.Widget_setMarginBottom)
    getMarginBottom = _swig_new_instance_method(_fifechan.Widget_getMarginBottom)
    setMarginLeft = _swig_new_instance_method(_fifechan.Widget_setMarginLeft)
    getMarginLeft = _swig_new_instance_method(_fifechan.Widget_getMarginLeft)
    setPadding = _swig_new_instance_method(_fifechan.Widget_setPadding)
    setPaddingTop = _swig_new_instance_method(_fifechan.Widget_setPaddingTop)
    getPaddingTop = _swig_new_instance_method(_fifechan.Widget_getPaddingTop)
    setPaddingRight = _swig_new_instance_method(_fifechan.Widget_setPaddingRight)
    getPaddingRight = _swig_new_instance_method(_fifechan.Widget_getPaddingRight)
    setPaddingBottom = _swig_new_instance_method(_fifechan.Widget_setPaddingBottom)
    getPaddingBottom = _swig_new_instance_method(_fifechan.Widget_getPaddingBottom)
    setPaddingLeft = _swig_new_instance_method(_fifechan.Widget_setPaddingLeft)
    getPaddingLeft = _swig_new_instance_method(_fifechan.Widget_getPaddingLeft)
    setFocusable = _swig_new_instance_method(_fifechan.Widget_setFocusable)
    isFocusable = _swig_new_instance_method(_fifechan.Widget_isFocusable)
    isFocused = _swig_new_instance_method(_fifechan.Widget_isFocused)
    setEnabled = _swig_new_instance_method(_fifechan.Widget_setEnabled)
    isEnabled = _swig_new_instance_method(_fifechan.Widget_isEnabled)
    setVisible = _swig_new_instance_method(_fifechan.Widget_setVisible)
    isVisible = _swig_new_instance_method(_fifechan.Widget_isVisible)
    isSetVisible = _swig_new_instance_method(_fifechan.Widget_isSetVisible)
    setBaseColor = _swig_new_instance_method(_fifechan.Widget_setBaseColor)
    getBaseColor = _swig_new_instance_method(_fifechan.Widget_getBaseColor)
    setForegroundColor = _swig_new_instance_method(_fifechan.Widget_setForegroundColor)
    getForegroundColor = _swig_new_instance_method(_fifechan.Widget_getForegroundColor)
    setBackgroundColor = _swig_new_instance_method(_fifechan.Widget_setBackgroundColor)
    getBackgroundColor = _swig_new_instance_method(_fifechan.Widget_getBackgroundColor)
    setSelectionColor = _swig_new_instance_method(_fifechan.Widget_setSelectionColor)
    getSelectionColor = _swig_new_instance_method(_fifechan.Widget_getSelectionColor)
    setOutlineColor = _swig_new_instance_method(_fifechan.Widget_setOutlineColor)
    getOutlineColor = _swig_new_instance_method(_fifechan.Widget_getOutlineColor)
    setBorderColor = _swig_new_instance_method(_fifechan.Widget_setBorderColor)
    getBorderColor = _swig_new_instance_method(_fifechan.Widget_getBorderColor)
    setSelectionMode = _swig_new_instance_method(_fifechan.Widget_setSelectionMode)
    getSelectionMode = _swig_new_instance_method(_fifechan.Widget_getSelectionMode)
    setMinSize = _swig_new_instance_method(_fifechan.Widget_setMinSize)
    getMinSize = _swig_new_instance_method(_fifechan.Widget_getMinSize)
    setMaxSize = _swig_new_instance_method(_fifechan.Widget_setMaxSize)
    getMaxSize = _swig_new_instance_method(_fifechan.Widget_getMaxSize)
    setFixedSize = _swig_new_instance_method(_fifechan.Widget_setFixedSize)
    getFixedSize = _swig_new_instance_method(_fifechan.Widget_getFixedSize)
    isFixedSize = _swig_new_instance_method(_fifechan.Widget_isFixedSize)
    setVerticalExpand = _swig_new_instance_method(_fifechan.Widget_setVerticalExpand)
    isVerticalExpand = _swig_new_instance_method(_fifechan.Widget_isVerticalExpand)
    setHorizontalExpand = _swig_new_instance_method(_fifechan.Widget_setHorizontalExpand)
    isHorizontalExpand = _swig_new_instance_method(_fifechan.Widget_isHorizontalExpand)
    isLayouted = _swig_new_instance_method(_fifechan.Widget_isLayouted)
    adaptLayout = _swig_new_instance_method(_fifechan.Widget_adaptLayout)
    resizeToContent = _swig_new_instance_method(_fifechan.Widget_resizeToContent)
    adjustSize = _swig_new_instance_method(_fifechan.Widget_adjustSize)
    expandContent = _swig_new_instance_method(_fifechan.Widget_expandContent)
    requestFocus = _swig_new_instance_method(_fifechan.Widget_requestFocus)
    requestMoveToTop = _swig_new_instance_method(_fifechan.Widget_requestMoveToTop)
    requestMoveToBottom = _swig_new_instance_method(_fifechan.Widget_requestMoveToBottom)
    setActionEventId = _swig_new_instance_method(_fifechan.Widget_setActionEventId)
    getActionEventId = _swig_new_instance_method(_fifechan.Widget_getActionEventId)
    getAbsolutePosition = _swig_new_instance_method(_fifechan.Widget_getAbsolutePosition)
    getFont = _swig_new_instance_method(_fifechan.Widget_getFont)
    setGlobalFont = _swig_new_static_method(_fifechan.Widget_setGlobalFont)
    setFont = _swig_new_instance_method(_fifechan.Widget_setFont)
    isTabInEnabled = _swig_new_instance_method(_fifechan.Widget_isTabInEnabled)
    setTabInEnabled = _swig_new_instance_method(_fifechan.Widget_setTabInEnabled)
    isTabOutEnabled = _swig_new_instance_method(_fifechan.Widget_isTabOutEnabled)
    setTabOutEnabled = _swig_new_instance_method(_fifechan.Widget_setTabOutEnabled)
    isModalFocusable = _swig_new_instance_method(_fifechan.Widget_isModalFocusable)
    isModalMouseInputFocusable = _swig_new_instance_method(_fifechan.Widget_isModalMouseInputFocusable)
    requestModalFocus = _swig_new_instance_method(_fifechan.Widget_requestModalFocus)
    requestModalMouseInputFocus = _swig_new_instance_method(_fifechan.Widget_requestModalMouseInputFocus)
    releaseModalFocus = _swig_new_instance_method(_fifechan.Widget_releaseModalFocus)
    releaseModalMouseInputFocus = _swig_new_instance_method(_fifechan.Widget_releaseModalMouseInputFocus)
    isModalFocused = _swig_new_instance_method(_fifechan.Widget_isModalFocused)
    isModalMouseInputFocused = _swig_new_instance_method(_fifechan.Widget_isModalMouseInputFocused)
    getWidgetAt = _swig_new_instance_method(_fifechan.Widget_getWidgetAt)
    moveToTop = _swig_new_instance_method(_fifechan.Widget_moveToTop)
    moveToBottom = _swig_new_instance_method(_fifechan.Widget_moveToBottom)
    focusNext = _swig_new_instance_method(_fifechan.Widget_focusNext)
    focusPrevious = _swig_new_instance_method(_fifechan.Widget_focusPrevious)
    addActionListener = _swig_new_instance_method(_fifechan.Widget_addActionListener)
    removeActionListener = _swig_new_instance_method(_fifechan.Widget_removeActionListener)
    addMouseListener = _swig_new_instance_method(_fifechan.Widget_addMouseListener)
    removeMouseListener = _swig_new_instance_method(_fifechan.Widget_removeMouseListener)
    addKeyListener = _swig_new_instance_method(_fifechan.Widget_addKeyListener)
    removeKeyListener = _swig_new_instance_method(_fifechan.Widget_removeKeyListener)
    addWidgetListener = _swig_new_instance_method(_fifechan.Widget_addWidgetListener)
    removeWidgetListener = _swig_new_instance_method(_fifechan.Widget_removeWidgetListener)
    draw = _swig_new_instance_method(_fifechan.Widget_draw)
    __swig_destroy__ = _fifechan.delete_Widget

# Register Widget in _fifechan:
_fifechan.Widget_swigregister(Widget)
class Spacer(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Spacer_swiginit(self, _fifechan.new_Spacer())
    __swig_destroy__ = _fifechan.delete_Spacer
    resizeToContent = _swig_new_instance_method(_fifechan.Spacer_resizeToContent)

# Register Spacer in _fifechan:
_fifechan.Spacer_swigregister(Spacer)
class Container(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Container_swiginit(self, _fifechan.new_Container())
    __swig_destroy__ = _fifechan.delete_Container
    setOpaque = _swig_new_instance_method(_fifechan.Container_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Container_isOpaque)
    add = _swig_new_instance_method(_fifechan.Container_add)
    remove = _swig_new_instance_method(_fifechan.Container_remove)
    clear = _swig_new_instance_method(_fifechan.Container_clear)
    Absolute = _fifechan.Container_Absolute
    AutoSize = _fifechan.Container_AutoSize
    Vertical = _fifechan.Container_Vertical
    Horizontal = _fifechan.Container_Horizontal
    Circular = _fifechan.Container_Circular
    setLayout = _swig_new_instance_method(_fifechan.Container_setLayout)
    getLayout = _swig_new_instance_method(_fifechan.Container_getLayout)
    setUniformSize = _swig_new_instance_method(_fifechan.Container_setUniformSize)
    isUniformSize = _swig_new_instance_method(_fifechan.Container_isUniformSize)
    setVerticalSpacing = _swig_new_instance_method(_fifechan.Container_setVerticalSpacing)
    getVerticalSpacing = _swig_new_instance_method(_fifechan.Container_getVerticalSpacing)
    setHorizontalSpacing = _swig_new_instance_method(_fifechan.Container_setHorizontalSpacing)
    getHorizontalSpacing = _swig_new_instance_method(_fifechan.Container_getHorizontalSpacing)
    setBackgroundWidget = _swig_new_instance_method(_fifechan.Container_setBackgroundWidget)
    getBackgroundWidget = _swig_new_instance_method(_fifechan.Container_getBackgroundWidget)

# Register Container in _fifechan:
_fifechan.Container_swigregister(Container)
class Button(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Button_swiginit(self, _fifechan.new_Button(*args))
    setCaption = _swig_new_instance_method(_fifechan.Button_setCaption)
    getCaption = _swig_new_instance_method(_fifechan.Button_getCaption)
    setActive = _swig_new_instance_method(_fifechan.Button_setActive)
    isActive = _swig_new_instance_method(_fifechan.Button_isActive)
    setAlignment = _swig_new_instance_method(_fifechan.Button_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.Button_getAlignment)
    setDownXOffset = _swig_new_instance_method(_fifechan.Button_setDownXOffset)
    getDownXOffset = _swig_new_instance_method(_fifechan.Button_getDownXOffset)
    setDownYOffset = _swig_new_instance_method(_fifechan.Button_setDownYOffset)
    getDownYOffset = _swig_new_instance_method(_fifechan.Button_getDownYOffset)
    setDownOffset = _swig_new_instance_method(_fifechan.Button_setDownOffset)
    adjustSize = _swig_new_instance_method(_fifechan.Button_adjustSize)
    __swig_destroy__ = _fifechan.delete_Button

# Register Button in _fifechan:
_fifechan.Button_swigregister(Button)
class ImageButton(Button):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ImageButton_swiginit(self, _fifechan.new_ImageButton(*args))
    __swig_destroy__ = _fifechan.delete_ImageButton
    setUpImage = _swig_new_instance_method(_fifechan.ImageButton_setUpImage)
    getUpImage = _swig_new_instance_method(_fifechan.ImageButton_getUpImage)
    setDownImage = _swig_new_instance_method(_fifechan.ImageButton_setDownImage)
    getDownImage = _swig_new_instance_method(_fifechan.ImageButton_getDownImage)
    setHoverImage = _swig_new_instance_method(_fifechan.ImageButton_setHoverImage)
    getHoverImage = _swig_new_instance_method(_fifechan.ImageButton_getHoverImage)
    setInactiveUpImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveUpImage)
    getInactiveUpImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveUpImage)
    setInactiveDownImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveDownImage)
    getInactiveDownImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveDownImage)
    setInactiveHoverImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveHoverImage)
    getInactiveHoverImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveHoverImage)
    resizeToContent = _swig_new_instance_method(_fifechan.ImageButton_resizeToContent)
    adjustSize = _swig_new_instance_method(_fifechan.ImageButton_adjustSize)

# Register ImageButton in _fifechan:
_fifechan.ImageButton_swigregister(ImageButton)
class ToggleButton(ImageButton):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ToggleButton_swiginit(self, _fifechan.new_ToggleButton(*args))
    __swig_destroy__ = _fifechan.delete_ToggleButton
    isSelected = _swig_new_instance_method(_fifechan.ToggleButton_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.ToggleButton_setSelected)
    toggleSelected = _swig_new_instance_method(_fifechan.ToggleButton_toggleSelected)
    setGroup = _swig_new_instance_method(_fifechan.ToggleButton_setGroup)
    getGroup = _swig_new_instance_method(_fifechan.ToggleButton_getGroup)

# Register ToggleButton in _fifechan:
_fifechan.ToggleButton_swigregister(ToggleButton)
class CheckBox(ImageButton):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Marker_Checkmark = _fifechan.CheckBox_Marker_Checkmark
    Marker_Cross = _fifechan.CheckBox_Marker_Cross
    Marker_Dot = _fifechan.CheckBox_Marker_Dot
    Marker_Rhombus = _fifechan.CheckBox_Marker_Rhombus
    Marker_Image = _fifechan.CheckBox_Marker_Image

    def __init__(self):
        _fifechan.CheckBox_swiginit(self, _fifechan.new_CheckBox())
    __swig_destroy__ = _fifechan.delete_CheckBox
    isSelected = _swig_new_instance_method(_fifechan.CheckBox_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.CheckBox_setSelected)
    toggleSelected = _swig_new_instance_method(_fifechan.CheckBox_toggleSelected)
    setBackgroundImage = _swig_new_instance_method(_fifechan.CheckBox_setBackgroundImage)
    getBackgroundImage = _swig_new_instance_method(_fifechan.CheckBox_getBackgroundImage)
    getMarkerStyle = _swig_new_instance_method(_fifechan.CheckBox_getMarkerStyle)
    setMarkerStyle = _swig_new_instance_method(_fifechan.CheckBox_setMarkerStyle)
    adjustSize = _swig_new_instance_method(_fifechan.CheckBox_adjustSize)

# Register CheckBox in _fifechan:
_fifechan.CheckBox_swigregister(CheckBox)
class Icon(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, image):
        _fifechan.Icon_swiginit(self, _fifechan.new_Icon(image))
    __swig_destroy__ = _fifechan.delete_Icon
    setImage = _swig_new_instance_method(_fifechan.Icon_setImage)
    getImage = _swig_new_instance_method(_fifechan.Icon_getImage)
    isScaling = _swig_new_instance_method(_fifechan.Icon_isScaling)
    setScaling = _swig_new_instance_method(_fifechan.Icon_setScaling)
    isTiling = _swig_new_instance_method(_fifechan.Icon_isTiling)
    setTiling = _swig_new_instance_method(_fifechan.Icon_setTiling)
    setOpaque = _swig_new_instance_method(_fifechan.Icon_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Icon_isOpaque)
    adjustSize = _swig_new_instance_method(_fifechan.Icon_adjustSize)

# Register Icon in _fifechan:
_fifechan.Icon_swigregister(Icon)
class ScrollArea(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ScrollArea_swiginit(self, _fifechan.new_ScrollArea(*args))
    __swig_destroy__ = _fifechan.delete_ScrollArea
    setContent = _swig_new_instance_method(_fifechan.ScrollArea_setContent)
    getContent = _swig_new_instance_method(_fifechan.ScrollArea_getContent)
    setHorizontalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setHorizontalScrollPolicy)
    getHorizontalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalScrollPolicy)
    setVerticalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setVerticalScrollPolicy)
    getVerticalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalScrollPolicy)
    setScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setScrollPolicy)
    setVerticalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setVerticalScrollAmount)
    getVerticalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalScrollAmount)
    setHorizontalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setHorizontalScrollAmount)
    getHorizontalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalScrollAmount)
    setScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setScrollAmount)
    getHorizontalMaxScroll = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalMaxScroll)
    getVerticalMaxScroll = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalMaxScroll)
    setScrollbarWidth = _swig_new_instance_method(_fifechan.ScrollArea_setScrollbarWidth)
    getScrollbarWidth = _swig_new_instance_method(_fifechan.ScrollArea_getScrollbarWidth)
    setLeftButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setLeftButtonScrollAmount)
    setRightButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setRightButtonScrollAmount)
    setUpButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setUpButtonScrollAmount)
    setDownButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setDownButtonScrollAmount)
    getLeftButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getLeftButtonScrollAmount)
    getRightButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getRightButtonScrollAmount)
    getUpButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getUpButtonScrollAmount)
    getDownButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getDownButtonScrollAmount)
    ShowAlways = _fifechan.ScrollArea_ShowAlways
    ShowNever = _fifechan.ScrollArea_ShowNever
    ShowAuto = _fifechan.ScrollArea_ShowAuto

# Register ScrollArea in _fifechan:
_fifechan.ScrollArea_swigregister(ScrollArea)
class ListModel(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_ListModel
    getNumberOfElements = _swig_new_instance_method(_fifechan.ListModel_getNumberOfElements)
    getElementAt = _swig_new_instance_method(_fifechan.ListModel_getElementAt)

    def __init__(self):
        if self.__class__ == ListModel:
            _self = None
        else:
            _self = self
        _fifechan.ListModel_swiginit(self, _fifechan.new_ListModel(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_ListModel(self)
        return weakref.proxy(self)

# Register ListModel in _fifechan:
_fifechan.ListModel_swigregister(ListModel)
class ListBox(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ListBox_swiginit(self, _fifechan.new_ListBox(*args))
    __swig_destroy__ = _fifechan.delete_ListBox
    getSelected = _swig_new_instance_method(_fifechan.ListBox_getSelected)
    setSelected = _swig_new_instance_method(_fifechan.ListBox_setSelected)
    setListModel = _swig_new_instance_method(_fifechan.ListBox_setListModel)
    getListModel = _swig_new_instance_method(_fifechan.ListBox_getListModel)
    adjustSize = _swig_new_instance_method(_fifechan.ListBox_adjustSize)
    isWrappingEnabled = _swig_new_instance_method(_fifechan.ListBox_isWrappingEnabled)
    setWrappingEnabled = _swig_new_instance_method(_fifechan.ListBox_setWrappingEnabled)

# Register ListBox in _fifechan:
_fifechan.ListBox_swigregister(ListBox)
class DropDown(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, listModel=None, scrollArea=None, listBox=None):
        _fifechan.DropDown_swiginit(self, _fifechan.new_DropDown(listModel, scrollArea, listBox))
    __swig_destroy__ = _fifechan.delete_DropDown
    getSelected = _swig_new_instance_method(_fifechan.DropDown_getSelected)
    setSelected = _swig_new_instance_method(_fifechan.DropDown_setSelected)
    setListModel = _swig_new_instance_method(_fifechan.DropDown_setListModel)
    getListModel = _swig_new_instance_method(_fifechan.DropDown_getListModel)
    adjustHeight = _swig_new_instance_method(_fifechan.DropDown_adjustHeight)
    setBaseColor = _swig_new_instance_method(_fifechan.DropDown_setBaseColor)
    getBaseColor = _swig_new_instance_method(_fifechan.DropDown_getBaseColor)
    setForegroundColor = _swig_new_instance_method(_fifechan.DropDown_setForegroundColor)
    getForegroundColor = _swig_new_instance_method(_fifechan.DropDown_getForegroundColor)
    setBackgroundColor = _swig_new_instance_method(_fifechan.DropDown_setBackgroundColor)
    getBackgroundColor = _swig_new_instance_method(_fifechan.DropDown_getBackgroundColor)
    setSelectionColor = _swig_new_instance_method(_fifechan.DropDown_setSelectionColor)
    getSelectionColor = _swig_new_instance_method(_fifechan.DropDown_getSelectionColor)

# Register DropDown in _fifechan:
_fifechan.DropDown_swigregister(DropDown)
class RadioButton(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.RadioButton_swiginit(self, _fifechan.new_RadioButton(*args))
    __swig_destroy__ = _fifechan.delete_RadioButton
    isSelected = _swig_new_instance_method(_fifechan.RadioButton_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.RadioButton_setSelected)
    getCaption = _swig_new_instance_method(_fifechan.RadioButton_getCaption)
    setCaption = _swig_new_instance_method(_fifechan.RadioButton_setCaption)
    setGroup = _swig_new_instance_method(_fifechan.RadioButton_setGroup)
    getGroup = _swig_new_instance_method(_fifechan.RadioButton_getGroup)
    adjustSize = _swig_new_instance_method(_fifechan.RadioButton_adjustSize)

# Register RadioButton in _fifechan:
_fifechan.RadioButton_swigregister(RadioButton)
class Slider(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Slider_swiginit(self, _fifechan.new_Slider(*args))
    __swig_destroy__ = _fifechan.delete_Slider
    setScale = _swig_new_instance_method(_fifechan.Slider_setScale)
    getScaleStart = _swig_new_instance_method(_fifechan.Slider_getScaleStart)
    setScaleStart = _swig_new_instance_method(_fifechan.Slider_setScaleStart)
    getScaleEnd = _swig_new_instance_method(_fifechan.Slider_getScaleEnd)
    setScaleEnd = _swig_new_instance_method(_fifechan.Slider_setScaleEnd)
    getValue = _swig_new_instance_method(_fifechan.Slider_getValue)
    setValue = _swig_new_instance_method(_fifechan.Slider_setValue)
    setMarkerLength = _swig_new_instance_method(_fifechan.Slider_setMarkerLength)
    getMarkerLength = _swig_new_instance_method(_fifechan.Slider_getMarkerLength)
    setOrientation = _swig_new_instance_method(_fifechan.Slider_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.Slider_getOrientation)
    setStepLength = _swig_new_instance_method(_fifechan.Slider_setStepLength)
    getStepLength = _swig_new_instance_method(_fifechan.Slider_getStepLength)
    Horizontal = _fifechan.Slider_Horizontal
    Vertical = _fifechan.Slider_Vertical

# Register Slider in _fifechan:
_fifechan.Slider_swigregister(Slider)
class Window(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Window_swiginit(self, _fifechan.new_Window(*args))
    __swig_destroy__ = _fifechan.delete_Window
    setCaption = _swig_new_instance_method(_fifechan.Window_setCaption)
    getCaption = _swig_new_instance_method(_fifechan.Window_getCaption)
    setAlignment = _swig_new_instance_method(_fifechan.Window_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.Window_getAlignment)
    setTitleBarHeight = _swig_new_instance_method(_fifechan.Window_setTitleBarHeight)
    getTitleBarHeight = _swig_new_instance_method(_fifechan.Window_getTitleBarHeight)
    setMovable = _swig_new_instance_method(_fifechan.Window_setMovable)
    isMovable = _swig_new_instance_method(_fifechan.Window_isMovable)
    setOpaque = _swig_new_instance_method(_fifechan.Window_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Window_isOpaque)
    resizeToContent = _swig_new_instance_method(_fifechan.Window_resizeToContent)

# Register Window in _fifechan:
_fifechan.Window_swigregister(Window)
class AdjustingContainer(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.AdjustingContainer_swiginit(self, _fifechan.new_AdjustingContainer())
    __swig_destroy__ = _fifechan.delete_AdjustingContainer
    setNumberOfColumns = _swig_new_instance_method(_fifechan.AdjustingContainer_setNumberOfColumns)
    getNumberOfColumns = _swig_new_instance_method(_fifechan.AdjustingContainer_getNumberOfColumns)
    setColumnAlignment = _swig_new_instance_method(_fifechan.AdjustingContainer_setColumnAlignment)
    getColumnAlignment = _swig_new_instance_method(_fifechan.AdjustingContainer_getColumnAlignment)
    adjustContent = _swig_new_instance_method(_fifechan.AdjustingContainer_adjustContent)
    LEFT = _fifechan.AdjustingContainer_LEFT
    CENTER = _fifechan.AdjustingContainer_CENTER
    RIGHT = _fifechan.AdjustingContainer_RIGHT

# Register AdjustingContainer in _fifechan:
_fifechan.AdjustingContainer_swigregister(AdjustingContainer)
class FlowContainer(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.FlowContainer_swiginit(self, _fifechan.new_FlowContainer())
    __swig_destroy__ = _fifechan.delete_FlowContainer
    setAlignment = _swig_new_instance_method(_fifechan.FlowContainer_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.FlowContainer_getAlignment)
    adjustContent = _swig_new_instance_method(_fifechan.FlowContainer_adjustContent)
    Left = _fifechan.FlowContainer_Left
    Right = _fifechan.FlowContainer_Right
    Top = _fifechan.FlowContainer_Top
    Bottom = _fifechan.FlowContainer_Bottom
    Center = _fifechan.FlowContainer_Center

# Register FlowContainer in _fifechan:
_fifechan.FlowContainer_swigregister(FlowContainer)
class TextBox(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.TextBox_swiginit(self, _fifechan.new_TextBox(*args))
    setText = _swig_new_instance_method(_fifechan.TextBox_setText)
    getText = _swig_new_instance_method(_fifechan.TextBox_getText)
    getTextRow = _swig_new_instance_method(_fifechan.TextBox_getTextRow)
    setTextRow = _swig_new_instance_method(_fifechan.TextBox_setTextRow)
    getNumberOfRows = _swig_new_instance_method(_fifechan.TextBox_getNumberOfRows)
    getCaretPosition = _swig_new_instance_method(_fifechan.TextBox_getCaretPosition)
    setCaretPosition = _swig_new_instance_method(_fifechan.TextBox_setCaretPosition)
    getCaretRow = _swig_new_instance_method(_fifechan.TextBox_getCaretRow)
    setCaretRow = _swig_new_instance_method(_fifechan.TextBox_setCaretRow)
    getCaretColumn = _swig_new_instance_method(_fifechan.TextBox_getCaretColumn)
    setCaretColumn = _swig_new_instance_method(_fifechan.TextBox_setCaretColumn)
    setCaretRowColumn = _swig_new_instance_method(_fifechan.TextBox_setCaretRowColumn)
    scrollToCaret = _swig_new_instance_method(_fifechan.TextBox_scrollToCaret)
    isEditable = _swig_new_instance_method(_fifechan.TextBox_isEditable)
    setEditable = _swig_new_instance_method(_fifechan.TextBox_setEditable)
    addRow = _swig_new_instance_method(_fifechan.TextBox_addRow)
    isOpaque = _swig_new_instance_method(_fifechan.TextBox_isOpaque)
    setOpaque = _swig_new_instance_method(_fifechan.TextBox_setOpaque)
    __swig_destroy__ = _fifechan.delete_TextBox

# Register TextBox in _fifechan:
_fifechan.TextBox_swigregister(TextBox)
class TextField(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.TextField_swiginit(self, _fifechan.new_TextField(*args))
    __swig_destroy__ = _fifechan.delete_TextField
    setText = _swig_new_instance_method(_fifechan.TextField_setText)
    getText = _swig_new_instance_method(_fifechan.TextField_getText)
    adjustSize = _swig_new_instance_method(_fifechan.TextField_adjustSize)
    adjustHeight = _swig_new_instance_method(_fifechan.TextField_adjustHeight)
    setCaretPosition = _swig_new_instance_method(_fifechan.TextField_setCaretPosition)
    getCaretPosition = _swig_new_instance_method(_fifechan.TextField_getCaretPosition)

# Register TextField in _fifechan:
_fifechan.TextField_swigregister(TextField)
class PasswordField(TextField):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        _fifechan.PasswordField_swiginit(self, _fifechan.new_PasswordField(*args, **kwargs))
    __swig_destroy__ = _fifechan.delete_PasswordField

# Register PasswordField in _fifechan:
_fifechan.PasswordField_swigregister(PasswordField)
class IconProgressBar(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.IconProgressBar_swiginit(self, _fifechan.new_IconProgressBar(*args))
    __swig_destroy__ = _fifechan.delete_IconProgressBar
    setImage = _swig_new_instance_method(_fifechan.IconProgressBar_setImage)
    getImage = _swig_new_instance_method(_fifechan.IconProgressBar_getImage)
    setMaxIcons = _swig_new_instance_method(_fifechan.IconProgressBar_setMaxIcons)
    getMaxIcons = _swig_new_instance_method(_fifechan.IconProgressBar_getMaxIcons)
    setOrientation = _swig_new_instance_method(_fifechan.IconProgressBar_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.IconProgressBar_getOrientation)
    advance = _swig_new_instance_method(_fifechan.IconProgressBar_advance)
    reset = _swig_new_instance_method(_fifechan.IconProgressBar_reset)
    setIconCount = _swig_new_instance_method(_fifechan.IconProgressBar_setIconCount)
    getIconCount = _swig_new_instance_method(_fifechan.IconProgressBar_getIconCount)
    setOpaque = _swig_new_instance_method(_fifechan.IconProgressBar_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.IconProgressBar_isOpaque)
    HORIZONTAL = _fifechan.IconProgressBar_HORIZONTAL
    VERTICAL = _fifechan.IconProgressBar_VERTICAL

# Register IconProgressBar in _fifechan:
_fifechan.IconProgressBar_swigregister(IconProgressBar)
class ImageProgressBar(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ImageProgressBar_swiginit(self, _fifechan.new_ImageProgressBar(*args))
    __swig_destroy__ = _fifechan.delete_ImageProgressBar
    setBarImage = _swig_new_instance_method(_fifechan.ImageProgressBar_setBarImage)
    getBarImage = _swig_new_instance_method(_fifechan.ImageProgressBar_getBarImage)
    setForegroundImage = _swig_new_instance_method(_fifechan.ImageProgressBar_setForegroundImage)
    getForegroundImage = _swig_new_instance_method(_fifechan.ImageProgressBar_getForegroundImage)
    setMaxValue = _swig_new_instance_method(_fifechan.ImageProgressBar_setMaxValue)
    getMaxValue = _swig_new_instance_method(_fifechan.ImageProgressBar_getMaxValue)
    setValue = _swig_new_instance_method(_fifechan.ImageProgressBar_setValue)
    getValue = _swig_new_instance_method(_fifechan.ImageProgressBar_getValue)
    setOrientation = _swig_new_instance_method(_fifechan.ImageProgressBar_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.ImageProgressBar_getOrientation)
    setOpaque = _swig_new_instance_method(_fifechan.ImageProgressBar_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.ImageProgressBar_isOpaque)
    HORIZONTAL = _fifechan.ImageProgressBar_HORIZONTAL
    VERTICAL = _fifechan.ImageProgressBar_VERTICAL

# Register ImageProgressBar in _fifechan:
_fifechan.ImageProgressBar_swigregister(ImageProgressBar)
class Tab(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Tab_swiginit(self, _fifechan.new_Tab())
    __swig_destroy__ = _fifechan.delete_Tab
    setTabbedArea = _swig_new_instance_method(_fifechan.Tab_setTabbedArea)
    getTabbedArea = _swig_new_instance_method(_fifechan.Tab_getTabbedArea)

# Register Tab in _fifechan:
_fifechan.Tab_swigregister(Tab)
class TabbedArea(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.TabbedArea_swiginit(self, _fifechan.new_TabbedArea())
    __swig_destroy__ = _fifechan.delete_TabbedArea
    setOpaque = _swig_new_instance_method(_fifechan.TabbedArea_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.TabbedArea_isOpaque)
    addTab = _swig_new_instance_method(_fifechan.TabbedArea_addTab)
    removeTabWithIndex = _swig_new_instance_method(_fifechan.TabbedArea_removeTabWithIndex)
    removeTab = _swig_new_instance_method(_fifechan.TabbedArea_removeTab)
    getNumberOfTabs = _swig_new_instance_method(_fifechan.TabbedArea_getNumberOfTabs)
    isTabSelected = _swig_new_instance_method(_fifechan.TabbedArea_isTabSelected)
    setSelectedTab = _swig_new_instance_method(_fifechan.TabbedArea_setSelectedTab)
    getSelectedTabIndex = _swig_new_instance_method(_fifechan.TabbedArea_getSelectedTabIndex)
    getSelectedTab = _swig_new_instance_method(_fifechan.TabbedArea_getSelectedTab)
    setBaseColor = _swig_new_instance_method(_fifechan.TabbedArea_setBaseColor)
    setBackgroundWidget = _swig_new_instance_method(_fifechan.TabbedArea_setBackgroundWidget)
    getBackgroundWidget = _swig_new_instance_method(_fifechan.TabbedArea_getBackgroundWidget)
    setLayout = _swig_new_instance_method(_fifechan.TabbedArea_setLayout)
    getLayout = _swig_new_instance_method(_fifechan.TabbedArea_getLayout)
    setUniformSize = _swig_new_instance_method(_fifechan.TabbedArea_setUniformSize)
    isUniformSize = _swig_new_instance_method(_fifechan.TabbedArea_isUniformSize)
    setVerticalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_setVerticalSpacing)
    getVerticalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_getVerticalSpacing)
    setHorizontalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_setHorizontalSpacing)
    getHorizontalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_getHorizontalSpacing)

# Register TabbedArea in _fifechan:
_fifechan.TabbedArea_swigregister(TabbedArea)
class BarGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.BarGraph_swiginit(self, _fifechan.new_BarGraph(*args))
    __swig_destroy__ = _fifechan.delete_BarGraph
    setBarX = _swig_new_instance_method(_fifechan.BarGraph_setBarX)
    getBarX = _swig_new_instance_method(_fifechan.BarGraph_getBarX)
    setBarY = _swig_new_instance_method(_fifechan.BarGraph_setBarY)
    getBarY = _swig_new_instance_method(_fifechan.BarGraph_getBarY)
    setBarPosition = _swig_new_instance_method(_fifechan.BarGraph_setBarPosition)
    setBarWidth = _swig_new_instance_method(_fifechan.BarGraph_setBarWidth)
    getBarWidth = _swig_new_instance_method(_fifechan.BarGraph_getBarWidth)
    setBarHeight = _swig_new_instance_method(_fifechan.BarGraph_setBarHeight)
    getBarHeight = _swig_new_instance_method(_fifechan.BarGraph_getBarHeight)
    setBarSize = _swig_new_instance_method(_fifechan.BarGraph_setBarSize)
    setOpaque = _swig_new_instance_method(_fifechan.BarGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.BarGraph_isOpaque)

# Register BarGraph in _fifechan:
_fifechan.BarGraph_swigregister(BarGraph)
class CurveGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.CurveGraph_swiginit(self, _fifechan.new_CurveGraph(*args))
    __swig_destroy__ = _fifechan.delete_CurveGraph
    setPointVector = _swig_new_instance_method(_fifechan.CurveGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.CurveGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.CurveGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.CurveGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.CurveGraph_getThickness)
    setAutomaticControllPoints = _swig_new_instance_method(_fifechan.CurveGraph_setAutomaticControllPoints)
    isAutomaticControllPoints = _swig_new_instance_method(_fifechan.CurveGraph_isAutomaticControllPoints)
    setOpaque = _swig_new_instance_method(_fifechan.CurveGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.CurveGraph_isOpaque)

# Register CurveGraph in _fifechan:
_fifechan.CurveGraph_swigregister(CurveGraph)
class LineGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.LineGraph_swiginit(self, _fifechan.new_LineGraph(*args))
    __swig_destroy__ = _fifechan.delete_LineGraph
    setPointVector = _swig_new_instance_method(_fifechan.LineGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.LineGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.LineGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.LineGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.LineGraph_getThickness)
    setOpaque = _swig_new_instance_method(_fifechan.LineGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.LineGraph_isOpaque)

# Register LineGraph in _fifechan:
_fifechan.LineGraph_swigregister(LineGraph)
class PieGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.PieGraph_swiginit(self, _fifechan.new_PieGraph(*args))
    __swig_destroy__ = _fifechan.delete_PieGraph
    setCenterX = _swig_new_instance_method(_fifechan.PieGraph_setCenterX)
    setCenterY = _swig_new_instance_method(_fifechan.PieGraph_setCenterY)
    getCenterX = _swig_new_instance_method(_fifechan.PieGraph_getCenterX)
    getCenterY = _swig_new_instance_method(_fifechan.PieGraph_getCenterY)
    setCenter = _swig_new_instance_method(_fifechan.PieGraph_setCenter)
    getCenter = _swig_new_instance_method(_fifechan.PieGraph_getCenter)
    setRadius = _swig_new_instance_method(_fifechan.PieGraph_setRadius)
    getRadius = _swig_new_instance_method(_fifechan.PieGraph_getRadius)
    addSegment = _swig_new_instance_method(_fifechan.PieGraph_addSegment)
    clearSegments = _swig_new_instance_method(_fifechan.PieGraph_clearSegments)
    setOpaque = _swig_new_instance_method(_fifechan.PieGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.PieGraph_isOpaque)

# Register PieGraph in _fifechan:
_fifechan.PieGraph_swigregister(PieGraph)
class PointGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.PointGraph_swiginit(self, _fifechan.new_PointGraph(*args))
    __swig_destroy__ = _fifechan.delete_PointGraph
    setPointVector = _swig_new_instance_method(_fifechan.PointGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.PointGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.PointGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.PointGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.PointGraph_getThickness)
    setOpaque = _swig_new_instance_method(_fifechan.PointGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.PointGraph_isOpaque)

# Register PointGraph in _fifechan:
_fifechan.PointGraph_swigregister(PointGraph)
class FcnPointVector(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    iterator = _swig_new_instance_method(_fifechan.FcnPointVector_iterator)
    def __iter__(self):
        return self.iterator()
    __nonzero__ = _swig_new_instance_method(_fifechan.FcnPointVector___nonzero__)
    __bool__ = _swig_new_instance_method(_fifechan.FcnPointVector___bool__)
    __len__ = _swig_new_instance_method(_fifechan.FcnPointVector___len__)
    __getslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___getslice__)
    __setslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___setslice__)
    __delslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___delslice__)
    __delitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___delitem__)
    __getitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___getitem__)
    __setitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___setitem__)
    pop = _swig_new_instance_method(_fifechan.FcnPointVector_pop)
    append = _swig_new_instance_method(_fifechan.FcnPointVector_append)
    empty = _swig_new_instance_method(_fifechan.FcnPointVector_empty)
    size = _swig_new_instance_method(_fifechan.FcnPointVector_size)
    swap = _swig_new_instance_method(_fifechan.FcnPointVector_swap)
    begin = _swig_new_instance_method(_fifechan.FcnPointVector_begin)
    end = _swig_new_instance_method(_fifechan.FcnPointVector_end)
    rbegin = _swig_new_instance_method(_fifechan.FcnPointVector_rbegin)
    rend = _swig_new_instance_method(_fifechan.FcnPointVector_rend)
    clear = _swig_new_instance_method(_fifechan.FcnPointVector_clear)
    get_allocator = _swig_new_instance_method(_fifechan.FcnPointVector_get_allocator)
    pop_back = _swig_new_instance_method(_fifechan.FcnPointVector_pop_back)
    erase = _swig_new_instance_method(_fifechan.FcnPointVector_erase)

    def __init__(self, *args):
        _fifechan.FcnPointVector_swiginit(self, _fifechan.new_FcnPointVector(*args))
    push_back = _swig_new_instance_method(_fifechan.FcnPointVector_push_back)
    front = _swig_new_instance_method(_fifechan.FcnPointVector_front)
    back = _swig_new_instance_method(_fifechan.FcnPointVector_back)
    assign = _swig_new_instance_method(_fifechan.FcnPointVector_assign)
    resize = _swig_new_instance_method(_fifechan.FcnPointVector_resize)
    insert = _swig_new_instance_method(_fifechan.FcnPointVector_insert)
    reserve = _swig_new_instance_method(_fifechan.FcnPointVector_reserve)
    capacity = _swig_new_instance_method(_fifechan.FcnPointVector_capacity)
    __swig_destroy__ = _fifechan.delete_FcnPointVector

# Register FcnPointVector in _fifechan:
_fifechan.FcnPointVector_swigregister(FcnPointVector)

